Type: Package
Package: shiny
Title: Web Application Framework for R
Version: 1.14.0
Authors@R: c(
    person("Winston", "Chang", , "winston@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-1576-2126")),
    person("Joe", "Cheng", , "joe@posit.co", role = "aut"),
    person("JJ", "Allaire", , "jj@posit.co", role = "aut"),
    person("Carson", "Sievert", , "carson@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4958-2844")),
    person("Barret", "Schloerke", , "barret@posit.co", role = "aut",
           comment = c(ORCID = "0000-0001-9986-114X")),
    person("Garrick", "Aden-Buie", , "garrick@adenbuie.com", role = "aut",
           comment = c(ORCID = "0000-0002-7111-0077")),
    person("Yihui", "Xie", , "yihui@posit.co", role = "aut"),
    person("Jeff", "Allen", role = "aut"),
    person("Jonathan", "McPherson", , "jonathan@posit.co", role = "aut"),
    person("Alan", "Dipert", role = "aut"),
    person("Barbara", "Borges", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49")),
    person(, "jQuery Foundation", role = "cph",
           comment = "jQuery library and jQuery UI library"),
    person(, "jQuery contributors", role = c("ctb", "cph"),
           comment = "jQuery library; authors listed in inst/www/shared/jquery-AUTHORS.txt"),
    person(, "jQuery UI contributors", role = c("ctb", "cph"),
           comment = "jQuery UI library; authors listed in inst/www/shared/jqueryui/AUTHORS.txt"),
    person("Mark", "Otto", role = "ctb",
           comment = "Bootstrap library"),
    person("Jacob", "Thornton", role = "ctb",
           comment = "Bootstrap library"),
    person(, "Bootstrap contributors", role = "ctb",
           comment = "Bootstrap library"),
    person(, "Twitter, Inc", role = "cph",
           comment = "Bootstrap library"),
    person("Prem Nawaz", "Khan", role = "ctb",
           comment = "Bootstrap accessibility plugin"),
    person("Victor", "Tsaran", role = "ctb",
           comment = "Bootstrap accessibility plugin"),
    person("Dennis", "Lembree", role = "ctb",
           comment = "Bootstrap accessibility plugin"),
    person("Srinivasu", "Chakravarthula", role = "ctb",
           comment = "Bootstrap accessibility plugin"),
    person("Cathy", "O'Connor", role = "ctb",
           comment = "Bootstrap accessibility plugin"),
    person(, "PayPal, Inc", role = "cph",
           comment = "Bootstrap accessibility plugin"),
    person("Stefan", "Petre", role = c("ctb", "cph"),
           comment = "Bootstrap-datepicker library"),
    person("Andrew", "Rowls", role = c("ctb", "cph"),
           comment = "Bootstrap-datepicker library"),
    person("Brian", "Reavis", role = c("ctb", "cph"),
           comment = "selectize.js library"),
    person("Salmen", "Bejaoui", role = c("ctb", "cph"),
           comment = "selectize-plugin-a11y library"),
    person("Denis", "Ineshin", role = c("ctb", "cph"),
           comment = "ion.rangeSlider library"),
    person("Sami", "Samhuri", role = c("ctb", "cph"),
           comment = "Javascript strftime library"),
    person(, "SpryMedia Limited", role = c("ctb", "cph"),
           comment = "DataTables library"),
    person("Ivan", "Sagalaev", role = c("ctb", "cph"),
           comment = "highlight.js library"),
    person("R Core Team", role = c("ctb", "cph"),
           comment = "tar implementation from R")
  )
Description: Makes it incredibly easy to build interactive web
    applications with R. Automatic "reactive" binding between inputs and
    outputs and extensive prebuilt widgets make it possible to build
    beautiful, responsive, and powerful applications with minimal effort.
License: MIT + file LICENSE
URL: https://shiny.posit.co/, https://github.com/rstudio/shiny
BugReports: https://github.com/rstudio/shiny/issues
Depends: methods, R (>= 3.1.2)
Imports: bslib (>= 0.6.0), cachem (>= 1.1.0), cli, commonmark (>=
        2.0.0), fastmap (>= 1.1.1), fontawesome (>= 0.4.0), glue (>=
        1.3.2), grDevices, htmltools (>= 0.5.4), httpuv (>= 1.5.2),
        jsonlite (>= 0.9.16), later (>= 1.0.0), lifecycle (>= 0.2.0),
        mime (>= 0.3), otel, promises (>= 1.5.0), R6 (>= 2.0), rlang
        (>= 0.4.10), sourcetools, tools, utils, withr, xtable
Suggests: Cairo (>= 1.5-5), coro (>= 1.1.0), datasets, DT, dygraphs,
        future, ggplot2, knitr (>= 1.6), magrittr, markdown, mirai,
        otelsdk (>= 0.2.0), ragg, reactlog (>= 1.0.0), rmarkdown, sass,
        shinytest2, showtext, testthat (>= 3.2.1), watcher, yaml
Config/Needs/check: shinyjs
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
Collate: 'app-handle.R' 'globals.R' 'app-state.R' 'app_template.R'
        'bind-cache.R' 'bind-event.R' 'bookmark-state-local.R'
        'bookmark-state.R' 'bootstrap-deprecated.R'
        'bootstrap-layout.R' 'conditions.R' 'map.R' 'utils.R'
        'bootstrap.R' 'busy-indicators-spinners.R' 'busy-indicators.R'
        'cache-utils.R' 'deprecated.R' 'devmode.R' 'diagnose.R'
        'extended-task.R' 'fileupload.R' 'graph.R' 'reactives.R'
        'reactive-domains.R' 'history.R' 'hooks.R' 'html-deps.R'
        'image-interact-opts.R' 'image-interact.R' 'imageutils.R'
        'input-action.R' 'input-checkbox.R' 'input-checkboxgroup.R'
        'input-date.R' 'input-daterange.R' 'input-file.R'
        'input-numeric.R' 'input-password.R' 'input-radiobuttons.R'
        'input-select.R' 'input-slider.R' 'input-submit.R'
        'input-text.R' 'input-textarea.R' 'input-utils.R'
        'insert-tab.R' 'insert-ui.R' 'jqueryui.R' 'knitr.R'
        'middleware-shiny.R' 'middleware.R' 'timer.R' 'shiny.R'
        'mock-session.R' 'modal.R' 'modules.R' 'notifications.R'
        'otel-attr-srcref.R' 'otel-collect.R' 'otel-enable.R'
        'otel-error.R' 'otel-label.R' 'otel-reactive-update.R'
        'otel-session.R' 'otel-shiny.R' 'otel-with.R' 'priorityqueue.R'
        'progress.R' 'react.R' 'reexports.R' 'render-cached-plot.R'
        'render-plot.R' 'render-table.R' 'run-url.R' 'runapp.R'
        'serializers.R' 'server-input-handlers.R'
        'server-resource-paths.R' 'server.R' 'shiny-options.R'
        'shiny-package.R' 'shinyapp.R' 'shinyui.R' 'shinywrappers.R'
        'showcase.R' 'snapshot.R' 'staticimports.R' 'tar.R'
        'test-export.R' 'test-server.R' 'test.R' 'update-input.R'
        'utils-lang.R' 'utils-tags.R' 'version_bs_date_picker.R'
        'version_ion_range_slider.R' 'version_jquery.R'
        'version_jqueryui.R' 'version_selectize.R' 'version_strftime.R'
        'viewer.R'
NeedsCompilation: no
Packaged: 2026-06-18 18:59:37 UTC; cpsievert
Author: Winston Chang [aut] (ORCID: <https://orcid.org/0000-0002-1576-2126>),
  Joe Cheng [aut],
  JJ Allaire [aut],
  Carson Sievert [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4958-2844>),
  Barret Schloerke [aut] (ORCID: <https://orcid.org/0000-0001-9986-114X>),
  Garrick Aden-Buie [aut] (ORCID:
    <https://orcid.org/0000-0002-7111-0077>),
  Yihui Xie [aut],
  Jeff Allen [aut],
  Jonathan McPherson [aut],
  Alan Dipert [aut],
  Barbara Borges [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>),
  jQuery Foundation [cph] (jQuery library and jQuery UI library),
  jQuery contributors [ctb, cph] (jQuery library; authors listed in
    inst/www/shared/jquery-AUTHORS.txt),
  jQuery UI contributors [ctb, cph] (jQuery UI library; authors listed in
    inst/www/shared/jqueryui/AUTHORS.txt),
  Mark Otto [ctb] (Bootstrap library),
  Jacob Thornton [ctb] (Bootstrap library),
  Bootstrap contributors [ctb] (Bootstrap library),
  Twitter, Inc [cph] (Bootstrap library),
  Prem Nawaz Khan [ctb] (Bootstrap accessibility plugin),
  Victor Tsaran [ctb] (Bootstrap accessibility plugin),
  Dennis Lembree [ctb] (Bootstrap accessibility plugin),
  Srinivasu Chakravarthula [ctb] (Bootstrap accessibility plugin),
  Cathy O'Connor [ctb] (Bootstrap accessibility plugin),
  PayPal, Inc [cph] (Bootstrap accessibility plugin),
  Stefan Petre [ctb, cph] (Bootstrap-datepicker library),
  Andrew Rowls [ctb, cph] (Bootstrap-datepicker library),
  Brian Reavis [ctb, cph] (selectize.js library),
  Salmen Bejaoui [ctb, cph] (selectize-plugin-a11y library),
  Denis Ineshin [ctb, cph] (ion.rangeSlider library),
  Sami Samhuri [ctb, cph] (Javascript strftime library),
  SpryMedia Limited [ctb, cph] (DataTables library),
  Ivan Sagalaev [ctb, cph] (highlight.js library),
  R Core Team [ctb, cph] (tar implementation from R)
Maintainer: Carson Sievert <carson@posit.co>
Repository: CRAN
Date/Publication: 2026-06-21 14:10:08 UTC
Built: R 4.5.2; ; 2026-06-21 18:37:03 UTC; unix
