####
# Generated by `./tools/documentation/updateReexports.R`: do not edit by hand
# Please call `source('tools/documentation/updateReexports.R')` from the root folder to update`
####


# fastmap key_missing.Rd -------------------------------------------------------

#' @importFrom fastmap key_missing
#' @export
fastmap::key_missing

#' @importFrom fastmap is.key_missing
#' @export
fastmap::is.key_missing



# htmltools builder.Rd ---------------------------------------------------------

#' @importFrom htmltools tags
#' @export
htmltools::tags

#' @importFrom htmltools p
#' @export
htmltools::p

#' @importFrom htmltools h1
#' @export
htmltools::h1

#' @importFrom htmltools h2
#' @export
htmltools::h2

#' @importFrom htmltools h3
#' @export
htmltools::h3

#' @importFrom htmltools h4
#' @export
htmltools::h4

#' @importFrom htmltools h5
#' @export
htmltools::h5

#' @importFrom htmltools h6
#' @export
htmltools::h6

#' @importFrom htmltools a
#' @export
htmltools::a

#' @importFrom htmltools br
#' @export
htmltools::br

#' @importFrom htmltools div
#' @export
htmltools::div

#' @importFrom htmltools span
#' @export
htmltools::span

#' @importFrom htmltools pre
#' @export
htmltools::pre

#' @importFrom htmltools code
#' @export
htmltools::code

#' @importFrom htmltools img
#' @export
htmltools::img

#' @importFrom htmltools strong
#' @export
htmltools::strong

#' @importFrom htmltools em
#' @export
htmltools::em

#' @importFrom htmltools hr
#' @export
htmltools::hr

#' @importFrom htmltools tag
#' @export
htmltools::tag


# htmltools tagList.Rd ---------------------------------------------------------

#' @importFrom htmltools tagList
#' @export
htmltools::tagList


# htmltools tagAppendAttributes.Rd ---------------------------------------------

#' @importFrom htmltools tagAppendAttributes
#' @export
htmltools::tagAppendAttributes

#' @importFrom htmltools tagHasAttribute
#' @export
htmltools::tagHasAttribute

#' @importFrom htmltools tagGetAttribute
#' @export
htmltools::tagGetAttribute


# htmltools tagAppendChild.Rd --------------------------------------------------

#' @importFrom htmltools tagAppendChild
#' @export
htmltools::tagAppendChild

#' @importFrom htmltools tagAppendChildren
#' @export
htmltools::tagAppendChildren

#' @importFrom htmltools tagSetChildren
#' @export
htmltools::tagSetChildren


# htmltools HTML.Rd ------------------------------------------------------------

#' @importFrom htmltools HTML
#' @export
htmltools::HTML


# htmltools include.Rd ---------------------------------------------------------

#' @importFrom htmltools includeHTML
#' @export
htmltools::includeHTML

#' @importFrom htmltools includeText
#' @export
htmltools::includeText

#' @importFrom htmltools includeMarkdown
#' @export
htmltools::includeMarkdown

#' @importFrom htmltools includeCSS
#' @export
htmltools::includeCSS

#' @importFrom htmltools includeScript
#' @export
htmltools::includeScript


# htmltools singleton.Rd -------------------------------------------------------

#' @importFrom htmltools singleton
#' @export
htmltools::singleton

#' @importFrom htmltools is.singleton
#' @export
htmltools::is.singleton


# htmltools validateCssUnit.Rd -------------------------------------------------

#' @importFrom htmltools validateCssUnit
#' @export
htmltools::validateCssUnit


# htmltools htmlTemplate.Rd ----------------------------------------------------

#' @importFrom htmltools htmlTemplate
#' @export
htmltools::htmlTemplate


# htmltools suppressDependencies.Rd --------------------------------------------

#' @importFrom htmltools suppressDependencies
#' @export
htmltools::suppressDependencies


# htmltools withTags.Rd --------------------------------------------------------

#' @importFrom htmltools withTags
#' @export
htmltools::withTags
