# tabsetPanel() markup is correct Code default Output
a
b
c
--- Code pills Output
a
b
c
--- Code bslib_tags(x) Output
a
b
c
--- Code bslib_tags(x) Output
a
b
c
--- Code dropdown_active Output
a
b
c
# navbarPage() markup is correct Code nav_page Output
a
b
c
--- Code bslib_tags(x) Output
a
b
c
# String input is handled properly Code nav_list Output
a
b
c
--- Code bslib_tags(x) Output
a
b
c
# Shiny.tag input produces a warning Code tab_tags Output
A div
a
b
c