% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinywrappers.R
\name{reactivePlot}
\alias{reactivePlot}
\title{Plot output (deprecated)}
\usage{
reactivePlot(func, width = "auto", height = "auto", ...)
}
\arguments{
\item{func}{A function.}

\item{width}{Width.}

\item{height}{Height.}

\item{...}{Other arguments to pass on.}
}
\description{
See \code{\link{renderPlot}}.
}
