% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinywrappers.R
\name{reactivePrint}
\alias{reactivePrint}
\title{Print output (deprecated)}
\usage{
reactivePrint(func)
}
\arguments{
\item{func}{A function.}
}
\description{
See \code{\link{renderPrint}}.
}
