% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/app.R
\name{knitr_methods}
\alias{knitr_methods}
\alias{knit_print.shiny.appobj}
\alias{knit_print.shiny.render.function}
\alias{knit_print.reactive}
\title{Knitr S3 methods}
\usage{
knit_print.shiny.appobj(x, ...)

knit_print.shiny.render.function(x, ..., inline = FALSE)

knit_print.reactive(x, ..., inline = FALSE)
}
\arguments{
\item{x}{Object to knit_print}

\item{...}{Additional knit_print arguments}

\item{inline}{Whether the object is printed inline.}
}
\description{
These S3 methods are necessary to help Shiny applications and UI chunks embed
themselves in knitr/rmarkdown documents.
}
