% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinywrappers.R
\name{reactiveText}
\alias{reactiveText}
\title{Text output (deprecated)}
\usage{
reactiveText(func)
}
\arguments{
\item{func}{A function.}
}
\description{
See \code{\link{renderText}}.
}
