% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinywrappers.R
\name{reactiveUI}
\alias{reactiveUI}
\title{UI output (deprecated)}
\usage{
reactiveUI(func)
}
\arguments{
\item{func}{A function.}
}
\description{
See \code{\link{renderUI}}.
}
